﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Controls.Charts;
using Infragistics.Windows;
using System.Windows.Media.Effects;
using System.Diagnostics;
using System.ComponentModel;
using System.Windows.Threading;

namespace CustomCrossHair
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        LiveDataViewModel vm = new LiveDataViewModel();
        public MainWindow()
        {
            InitializeComponent();
            DataContext = vm;

            CustomRangeCrossHairs crch = CustomCrossHairBehavior.GetCustomCrossHair(xamDataChart1);

            startDateText.SetBinding(
                TextBlock.TextProperty,
                new Binding
                {
                    Source = crch,
                    Path = new PropertyPath("StartDateTimeValue"),
                    StringFormat = "dd MMM yyyy",
                });
            endDateText.SetBinding(
                TextBlock.TextProperty,
                new Binding
                {
                    Source = crch,
                    Path = new PropertyPath("EndDateTimeValue"),
                    StringFormat = "dd MMM yyyy",
                });

            startDoubleText.SetBinding(
                TextBlock.TextProperty,
                new Binding
                {
                    Source = crch,
                    Path = new PropertyPath("StartDoubleValue"),
                    StringFormat = "N2",
                });
            endDoubleText.SetBinding(
                TextBlock.TextProperty,
                new Binding
                {
                    Source = crch,
                    Path = new PropertyPath("EndDoubleValue"),
                    StringFormat = "N2",
                });

        }

        private void XamSliderNumericThumb_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            CustomCrossHairBehavior.GetCustomCrossHair(xamDataChart1).VerticalCrosshairRange = e.NewValue;
        }

        private void XamSliderNumericThumb_ValueChanged_1(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            CustomCrossHairBehavior.GetCustomCrossHair(xamDataChart1).HorizontalCrosshairRange = new TimeSpan((long)System.Convert.ToUInt64(e.NewValue)); ;
        }

        private void Vertical_Visibility(object sender, RoutedEventArgs e)
        {
            CustomCrossHairBehavior.GetCustomCrossHair(xamDataChart1).VerticalCrossHairSettings.Visibility = (bool)(sender as CheckBox).IsChecked ? Visibility.Visible : Visibility.Collapsed;
        }

        private void Horizontal_Visibility(object sender, RoutedEventArgs e)
        {
            CustomCrossHairBehavior.GetCustomCrossHair(xamDataChart1).HorizontalCrossHairSettings.Visibility = (bool)(sender as CheckBox).IsChecked ? Visibility.Visible : Visibility.Collapsed;
        }

        private void start_Click(object sender, RoutedEventArgs e)
        {
            start.IsEnabled = false;
            stop.IsEnabled = true;
            vm.Start();
        }

        private void stop_Click(object sender, RoutedEventArgs e)
        {
            start.IsEnabled = true;
            stop.IsEnabled = false;
            vm.Start();
        }
    }

    public partial class CustomCrossHairBehavior : DependencyObject
    {
        public static readonly DependencyProperty CustomCrossHairProperty =
            DependencyProperty.RegisterAttached("CustomCrossHair",
            typeof(CustomRangeCrossHairs), typeof(CustomCrossHairBehavior),
            new PropertyMetadata(null,
                (o, e) =>
                {
                    CustomCrossHairChanged(o as XamDataChart,
                        e.OldValue as CustomRangeCrossHairs,
                        e.NewValue as CustomRangeCrossHairs);

                }));

        public static CustomRangeCrossHairs GetCustomCrossHair(DependencyObject target)
        {
            return target.GetValue(CustomCrossHairProperty) as CustomRangeCrossHairs;
        }

        public static void SetCustomCrossHair(DependencyObject target, CustomRangeCrossHairs value)
        {
            target.SetValue(CustomCrossHairProperty, value);
        }

        public static void CustomCrossHairChanged(
            XamDataChart chart,
            CustomRangeCrossHairs oldVal,
            CustomRangeCrossHairs newVal)
        {
            if (chart == null)
            {
                return;
            }
            if (oldVal != null)
            {
                oldVal.DetachOwner(chart);
            }
            if (newVal != null)
            {
                newVal.AttachOwner(chart);
            }
        }

    }

    public class CustomRangeCrossHairs : DependencyObject, INotifyPropertyChanged
    {
        #region Private Fields
        private Canvas _container;
        private XamDataChart _owner;
        private Panel _lastMarkerContainer;
        private object _currentData;
        private Path _horizontalStart;
        private Path _horizontalEnd;
        private Path _verticalStart;
        private Path _verticalEnd;
        private DateTime startDateValue;
        private DateTime endDateValue;
        private double startDoubleValue;
        private double endDoubleValue;
        #endregion

        #region Initializing methods
        public void AttachOwner(XamDataChart chart)
        {
            if (_owner != null)
            {
                DetachOwner(_owner);
            }

            _owner = chart;           
            _owner.LayoutUpdated += _owner_LayoutUpdated;
            _owner.MouseMove += _owner_MouseMove;
            _owner.MouseEnter += _owner_MouseEnter;
            _owner.MouseLeave += _owner_MouseLeave;
           

            _container = new Canvas();
            Panel.SetZIndex(_container, 1000);
            _owner.Dispatcher.BeginInvoke(new Action(() =>
                {
                    
                    Border chartContent =
                        Utilities.GetDescendantFromType(
                         Utilities.GetDescendantFromName(_owner, "ContentPresenter"), typeof(Border), false) as Border;
                    (chartContent.Child as Grid).Children.Add(_container);

                    _horizontalStart = new Path();
                    _horizontalEnd = new Path();
                    InitializePathBindings(_horizontalStart, true);
                    InitializePathBindings(_horizontalEnd, true);


                    _verticalStart = new Path();
                    _verticalEnd = new Path();
                    InitializePathBindings(_verticalStart, false);
                    InitializePathBindings(_verticalEnd, false);


                    _container.Children.Add(_verticalStart);
                    _container.Children.Add(_verticalEnd);
                    _container.Children.Add(_horizontalStart);
                    _container.Children.Add(_horizontalEnd);

                }), System.Windows.Threading.DispatcherPriority.Background, null);        

        }





        public void DetachOwner(XamDataChart chart)
        {
            if (_owner != chart)
            {
                return;
            }

            chart.MouseMove -= _owner_MouseMove;
            chart.MouseLeave -= _owner_MouseLeave;
            chart.MouseEnter -= _owner_MouseEnter;
            chart.LayoutUpdated -= _owner_LayoutUpdated;

            Grid parent = _container.Parent as Grid;
            parent.Children.Remove(_container);

            _owner = null;
        }

        private void InitializePathBindings(Path path, bool isHorizontal)
        {
            string prop = isHorizontal ? "HorizontalCrossHairSettings." : "VerticalCrossHairSettings.";
            path.SetBinding(Path.StrokeProperty,
                 new Binding
                 {
                     Source = this,
                     Path = new PropertyPath(prop + "Stroke")
                 });
            path.SetBinding(Path.StrokeDashArrayProperty,
                 new Binding
                 {
                     Source = this,
                     Path = new PropertyPath(prop + "DashArray")
                 });
            path.SetBinding(Path.StrokeThicknessProperty,
                 new Binding
                 {
                     Source = this,
                     Path = new PropertyPath(prop + "Thickness")
                 });
            path.SetBinding(Path.EffectProperty,
                 new Binding
                 {
                     Source = this,
                     Path = new PropertyPath(prop + "Effect")
                 });
            path.SetBinding(Path.VisibilityProperty,
                 new Binding
                 {
                     Source = this,
                     Path = new PropertyPath(prop + "Visibility")
                 });
        }
        #endregion  

        #region Event Handlers
        void _owner_LayoutUpdated(object sender, EventArgs e)
        {        
            Refresh();
        }

        void _owner_MouseMove(object sender, MouseEventArgs e)
        {
            Marker marker = Utilities.GetAncestorFromType(e.OriginalSource as DependencyObject, typeof(Marker), false) as Marker;
            if (marker != null)
            {
                _lastMarkerContainer = marker.Parent as Panel;
                _currentData = (marker.Content as DataContext).Item;
                Refresh();
            }
        }

        void _owner_MouseEnter(object sender, MouseEventArgs e)
        {
            if (_verticalStart == null 
                || _horizontalStart == null
                || _verticalEnd == null
                || _horizontalEnd == null)
                return;
            _verticalStart.Opacity = 1;
            _verticalEnd.Opacity = 1;
            _horizontalStart.Opacity = 1;
            _horizontalEnd.Opacity = 1;
        }

        void _owner_MouseLeave(object sender, MouseEventArgs e)
        {
            if (_verticalStart == null
                 || _horizontalStart == null
                 || _verticalEnd == null
                 || _horizontalEnd == null)
                return;
            _verticalStart.Opacity = 0;
            _verticalEnd.Opacity = 0;
            _horizontalStart.Opacity = 0;
            _horizontalEnd.Opacity = 0;
        }
        #endregion

        #region Helper Methods
        private void Refresh()
        {
            if (_verticalStart == null || _horizontalStart == null)
                return;

            Point mousePoint = Mouse.GetPosition(_container);

            CategoryDateTimeXAxis xAxis = _owner.Axes.Where(axis => axis is CategoryDateTimeXAxis).FirstOrDefault() as CategoryDateTimeXAxis;
            NumericYAxis yAxis = _owner.Axes.Where(axis => axis is NumericYAxis).FirstOrDefault() as NumericYAxis;

            var viewport = new Rect(0, 0, xAxis.ActualWidth, yAxis.ActualHeight);
            var window = _owner.WindowRect;

            ScalerParams sParams = new ScalerParams(window, viewport, yAxis.IsInverted);

            double xVal = xAxis.GetUnscaledValue(mousePoint.X, sParams);
            DateTime xDate = new DateTime((long)Convert.ToUInt64(xVal));
            double yVal = yAxis.GetUnscaledValue(mousePoint.Y, sParams);

            #region Range X Coordinates
            DateTime startXValue = xDate.AddTicks(-1 * HorizontalCrosshairRange.Ticks / 2);
            DateTime endXValue = xDate.AddTicks(HorizontalCrosshairRange.Ticks / 2);

            StartDateTimeValue = startXValue;
            EndDateTimeValue = endXValue;

            double startXCoortinate = xAxis.GetScaledValue(startXValue.Ticks, sParams);
            double endXCoortinate = xAxis.GetScaledValue(endXValue.Ticks, sParams);
            #endregion

            #region Range Y Coordinates
            double startYValue = yVal - (double)(VerticalCrosshairRange / 2);
            double endYValue = yVal + (VerticalCrosshairRange / 2);

            StartDoubleValue = startYValue;
            EndDoubleValue = endYValue;

            double startYCoortinate = yAxis.GetScaledValue(startYValue, sParams);
            double endYCoortinate = yAxis.GetScaledValue(endYValue, sParams);
            #endregion

            _verticalStart.Data = new LineGeometry(
                new Point(startXCoortinate, 0),
                new Point(startXCoortinate, _container.ActualHeight));

            _verticalEnd.Data = new LineGeometry(
              new Point(endXCoortinate, 0),
              new Point(endXCoortinate, _container.ActualHeight));

            _horizontalStart.Data = new LineGeometry(
                new Point(0, startYCoortinate),
                new Point(_container.ActualWidth, startYCoortinate));

            _horizontalEnd.Data = new LineGeometry(
                new Point(0, endYCoortinate),
                new Point(_container.ActualWidth, endYCoortinate));
        }
        #endregion

        #region Properties

        #region Horizontal Settings
        public CustomCrossHairSettings HorizontalCrossHairSettings
        {
            get { return (CustomCrossHairSettings)GetValue(HorizontalCrossHairSettingsProperty); }
            set { SetValue(HorizontalCrossHairSettingsProperty, value); }
        }

        // Using a DependencyProperty as the backing store for HorizontalCrossHairSettings.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty HorizontalCrossHairSettingsProperty =
            DependencyProperty.Register("HorizontalCrossHairSettings",
            typeof(CustomCrossHairSettings), typeof(CustomRangeCrossHairs),
            new PropertyMetadata(null));
        #endregion

        #region Vertical Settings
        public CustomCrossHairSettings VerticalCrossHairSettings
        {
            get { return (CustomCrossHairSettings)GetValue(VerticalCrossHairSettingsProperty); }
            set { SetValue(VerticalCrossHairSettingsProperty, value); }
        }

        // Using a DependencyProperty as the backing store for VerticalCrossHairSettings.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty VerticalCrossHairSettingsProperty =
            DependencyProperty.Register("VerticalCrossHairSettings",
            typeof(CustomCrossHairSettings), typeof(CustomRangeCrossHairs), 
            new PropertyMetadata(null));
        #endregion

        #region Horizontal Range
        public TimeSpan HorizontalCrosshairRange
        {
            get { return (TimeSpan)GetValue(HorizontalCrosshairRangeProperty); }
            set { SetValue(HorizontalCrosshairRangeProperty, value); }
        }

        // Using a DependencyProperty as the backing store for HorizontalCrosshairRange.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty HorizontalCrosshairRangeProperty =
            DependencyProperty.Register("HorizontalCrosshairRange", typeof(TimeSpan), typeof(CustomRangeCrossHairs), new UIPropertyMetadata(TimeSpan.FromHours(0)));
        #endregion

        #region Vertical Range
        public double VerticalCrosshairRange
        {
            get { return (double)GetValue(VerticalCrosshairRangProperty); }
            set { SetValue(VerticalCrosshairRangProperty, value); }
        }

        // Using a DependencyProperty as the backing store for VerticalCrosshairRang.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty VerticalCrosshairRangProperty =
            DependencyProperty.Register("VerticalCrosshairRang", typeof(double), typeof(CustomRangeCrossHairs), new UIPropertyMetadata(0.0));
        #endregion

        #region Value Properties
        public DateTime StartDateTimeValue
        {
            get { return startDateValue; }
            set { startDateValue = value; OnPropertyChanged("StartDateTimeValue"); }
        }

        public DateTime EndDateTimeValue
        {
            get { return endDateValue; }
            set { endDateValue = value; OnPropertyChanged("EndDateTimeValue"); }
        }

        public double StartDoubleValue
        {
            get { return startDoubleValue; }
            set { startDoubleValue = value; OnPropertyChanged("StartDoubleValue"); }
        }

        public double EndDoubleValue
        {
            get { return endDoubleValue; }
            set { endDoubleValue = value; OnPropertyChanged("EndDoubleValue"); }
        }
        #endregion         
        #endregion

        #region INotifyPropertyChanged Members
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
        #endregion
    }

    public class CustomCrossHairSettings : DependencyObject
    {

        public double Thickness
        {
            get { return (double)GetValue(ThicknessProperty); }
            set { SetValue(ThicknessProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Thickness.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ThicknessProperty =
            DependencyProperty.Register("Thickness", typeof(double), typeof(CustomCrossHairSettings), new UIPropertyMetadata(1.0));




        public Brush Stroke
        {
            get { return (Brush)GetValue(StrokeProperty); }
            set { SetValue(StrokeProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Stroke.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty StrokeProperty =
            DependencyProperty.Register("Stroke", typeof(Brush), typeof(CustomCrossHairSettings), new UIPropertyMetadata(Brushes.Gray));





        public DoubleCollection DashArray
        {
            get { return (DoubleCollection)GetValue(DashArrayProperty); }
            set { SetValue(DashArrayProperty, value); }
        }

        // Using a DependencyProperty as the backing store for DashArray.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty DashArrayProperty =
            DependencyProperty.Register("DashArray", typeof(DoubleCollection), typeof(CustomCrossHairSettings), new UIPropertyMetadata(null));




        public Effect Effect
        {
            get { return (Effect)GetValue(EffectProperty); }
            set { SetValue(EffectProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Effect.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty EffectProperty =
            DependencyProperty.Register("Effect", typeof(Effect), typeof(CustomCrossHairSettings), new UIPropertyMetadata(null));



        public Visibility Visibility
        {
            get { return (Visibility)GetValue(VisibilityProperty); }
            set { SetValue(VisibilityProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Visibility.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty VisibilityProperty =
            DependencyProperty.Register("Visibility", typeof(Visibility), typeof(CustomCrossHairSettings), new UIPropertyMetadata(Visibility.Visible));



    }

    public class ChartData
    {
        public DateTime Date { get; set; }
        public double Value1 { get; set; }
        public double Value2 { get; set; }
        public double Value3 { get; set; }

        public static ObservableCollection<ChartData> GenerateData(int count)
        {
            ObservableCollection<ChartData> data = new ObservableCollection<ChartData>();

            Random r = new Random();

            ChartData data0 = new ChartData
            {
                Date = DateTime.Now,
                Value1 = r.Next(20, 40),
                Value2 = r.Next(20, 40),
                Value3 = r.Next(20, 40),
            };

            data.Add(data0);

            for (int i = 1; i < count; i++)
            {
                ChartData dat = new ChartData
                {
                    Date = DateTime.Now.AddDays(i),
                };

                if (r.NextDouble() > .5)
                {
                    dat.Value1 = data[i - 1].Value1 + r.NextDouble();
                    dat.Value2 = data[i - 1].Value2 + r.NextDouble();
                    dat.Value3 = data[i - 1].Value3 + r.NextDouble();
                }
                else
                {
                    dat.Value1 = data[i - 1].Value1 - r.NextDouble();
                    dat.Value2 = data[i - 1].Value2 - r.NextDouble();
                    dat.Value3 = data[i - 1].Value3 - r.NextDouble();
                }

                data.Add(dat);
            }

            return data;
        }
    }

    public class LiveDataViewModel
    {
        private ObservableCollection<ChartData> data;
        DispatcherTimer updateThread;

        public LiveDataViewModel()
        {
            Data = ChartData.GenerateData(1000);

            updateThread = new DispatcherTimer();
            updateThread.Interval = new TimeSpan(0, 0, 0, 0, 100);

            updateThread.Tick += (s, e) =>
            {

                Random r = new Random();


                ChartData dat = new ChartData
                {
                    Date = Data.Last().Date.AddDays(1),
                };

                if (r.NextDouble() > .5)
                {
                    dat.Value1 = Data.Last().Value1 + r.NextDouble();
                    dat.Value2 = Data.Last().Value2 + r.NextDouble();
                    dat.Value3 = Data.Last().Value3 + r.NextDouble();
                }
                else
                {
                    dat.Value1 = Data.Last().Value1 - r.NextDouble();
                    dat.Value2 = Data.Last().Value2 - r.NextDouble();
                    dat.Value3 = Data.Last().Value3 - r.NextDouble();
                }

                Data.RemoveAt(0);
                Data.Add(dat);
            };           
        }

        public void Start()
        {
            updateThread.Start();
        }

        public void Stop()
        {
            updateThread.Stop();
        }
        public ObservableCollection<ChartData> Data
        {
            get { return data; }
            set { data = value; }
        }
    }

    public class ThicksToTimeSpanConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            TimeSpan time = new TimeSpan((long)System.Convert.ToUInt64(value));
            return time.ToString("%d") + " days " +  time.ToString(@"hh\:mm\:ss"); 
;
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            return ((TimeSpan)value).Ticks;
        }
    }

}
